@@ -63,7 +63,7 @@ class SimulatorTabController(QWidget):
         util.set_splitter_stylesheet(self.ui.splitter)
         util.set_splitter_stylesheet(self.ui.splitterLeftRight)
 
-        self.ui.splitter.setSizes([self.width() / 0.7, self.width() / 0.3])
+        self.ui.splitter.setSizes([int(self.width() / 0.7), int(self.width() / 0.3)])
 
         self.ui.treeProtocols.setHeaderHidden(True)
         self.tree_model = self.generator_tab_controller.tree_model
@@ -120,7 +120,7 @@ class SimulatorTabController(QWidget):
         frame.layout().setContentsMargins(0, 0, 0, 0)
         self.ui.tabWidget.setCornerWidget(frame)
 
-        self.ui.splitterLeftRight.setSizes([0.2 * self.width(), 0.8 * self.width()])
+        self.ui.splitterLeftRight.setSizes([int(0.2 * self.width()), int(0.8 * self.width())])
 
         self.create_connects()
 
@@ -276,7 +276,7 @@ class SimulatorTabController(QWidget):
         modulator_dialog = ModulatorDialog(self.project_manager.modulators, tree_model=self.tree_model, parent=self)
         modulator_dialog.ui.comboBoxCustomModulations.setCurrentIndex(preselected_index)
         modulator_dialog.showMaximized()
-        modulator_dialog.initialize(selected_message.encoded_bits_str[0:10])
+        modulator_dialog.initialize(selected_message.encoded_bits_str[0:16])
 
         modulator_dialog.finished.connect(self.refresh_modulators)
         modulator_dialog.finished.connect(self.generator_tab_controller.refresh_pause_list)
