@@ -90,7 +90,7 @@ class Ui_FormDeviceSettings(object):
         self.labelGain.setObjectName("labelGain")
         self.gridLayout.addWidget(self.labelGain, 10, 0, 1, 1)
         self.comboBoxDeviceIdentifier = QtWidgets.QComboBox(self.frame_2)
-        self.comboBoxDeviceIdentifier.setEditable(False)
+        self.comboBoxDeviceIdentifier.setEditable(True)
         self.comboBoxDeviceIdentifier.setInsertPolicy(QtWidgets.QComboBox.NoInsert)
         self.comboBoxDeviceIdentifier.setObjectName("comboBoxDeviceIdentifier")
         self.gridLayout.addWidget(self.comboBoxDeviceIdentifier, 1, 1, 1, 1)
@@ -288,14 +288,17 @@ class Ui_FormDeviceSettings(object):
         self.labelFreqCorrection.setText(_translate("FormDeviceSettings", "Frequency correction:"))
         self.labelGain.setToolTip(_translate("FormDeviceSettings", "<html><head/><body><p>The gain (more exactly RF gain) is the gain applied to the RF signal. This amplifies the high frequent signal arriving at the antenna of your Software Defined Radio.</p></body></html>"))
         self.labelGain.setText(_translate("FormDeviceSettings", "Gain:"))
+        self.comboBoxDeviceIdentifier.setToolTip(_translate("FormDeviceSettings", "<html><head/><body><p>You can enter a device identifier here if you have multiple SDRs of the same type attached to separate them. There is no need to configure this value otherwise. URH will <span style=\" font-weight:600;\">automatically</span> select an attached SDR of the configured type if you leave this value empty.</p></body></html>"))
         self.labelDCCorrection.setToolTip(_translate("FormDeviceSettings", "Apply DC correction during recording, that is, ensure the captured signal has a mean value of zero."))
         self.labelDCCorrection.setText(_translate("FormDeviceSettings", "DC correction:"))
+        self.labelDeviceIdentifier.setToolTip(_translate("FormDeviceSettings", "<html><head/><body><p>You can enter a device identifier here if you have multiple SDRs of the same type attached to separate them. There is no need to configure this value otherwise. URH will <span style=\" font-weight:600;\">automatically</span> select an attached SDR of the configured type if you leave this value empty.</p></body></html>"))
         self.labelDeviceIdentifier.setText(_translate("FormDeviceSettings", "Device Identifier:"))
         self.labelPort.setText(_translate("FormDeviceSettings", "Port number:"))
         self.labelIFGain.setToolTip(_translate("FormDeviceSettings", "<html><head/><body><p>The IF Gain is applied to the Intermediate Frequency signal in your Software Defined Radio. An IF signal has a lower frequency than the high frequent RF signal, so signal processing can be applied more efficiently.</p></body></html>"))
         self.labelIFGain.setText(_translate("FormDeviceSettings", "IF Gain:"))
         self.label_3.setText(_translate("FormDeviceSettings", "Device:"))
         self.labelAntenna.setText(_translate("FormDeviceSettings", "Antenna:"))
+        self.btnRefreshDeviceIdentifier.setToolTip(_translate("FormDeviceSettings", "<html><head/><body><p>Automatically detect connected SDRs of the above configured type. There is <span style=\" font-weight:600;\">no need</span> to press this button if you have only one SDR of a certain type attached. </p></body></html>"))
         self.btnRefreshDeviceIdentifier.setText(_translate("FormDeviceSettings", "..."))
         self.cbDevice.setItemText(0, _translate("FormDeviceSettings", "USRP"))
         self.cbDevice.setItemText(1, _translate("FormDeviceSettings", "HackRF"))
